Package: metalcor
Title: Meta-Analysis of Correlated Genetic Association Studies
Version: 1.0.0
Authors@R: c(
    person(given = "Alejandro",
	   family = "Ochoa",
	   role = c("aut", "cre"),
	   email = "alejandro.ochoa@duke.edu",
           comment = c(ORCID = "0000-0003-4928-3403")),
    person(given = "Tiffany",
	   family = "Tu",
	   role = c("aut"),
	   email = "tiffany.tu@duke.edu",
           comment = c(ORCID = "0000-0002-9357-6660"))
    )
Description: The main function performs meta-analysis of genetic association study summary statistics that may be correlated due to cryptic relatedness or other confounders, generalizing inverse variance weighted methods.  The function that estimates the correlation structure is also provided standalone.  Another key innovation, the estimation of the correlation parameter from the median product of correlated standard normal variables, is provided, as well as a complete set of functions for their underlying distribution: density, cumulative, quantile, and random deviates.  Described in Tu and Ochoa (2025) <doi:10.1101/2025.05.10.653279>.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 8.0.0
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Imports: stats, dplyr, tibble, tidyselect
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://github.com/OchoaLab/metalcor
BugReports: https://github.com/OchoaLab/metalcor/issues
NeedsCompilation: no
Packaged: 2026-05-06 19:25:54 UTC; viiia
Author: Alejandro Ochoa [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4928-3403>),
  Tiffany Tu [aut] (ORCID: <https://orcid.org/0000-0002-9357-6660>)
Maintainer: Alejandro Ochoa <alejandro.ochoa@duke.edu>
Repository: CRAN
Date/Publication: 2026-05-12 19:00:02 UTC
Built: R 4.5.2; ; 2026-05-12 20:51:29 UTC; unix
